# GNU MediaGoblin -- federated, autonomous media hosting
# Copyright (C) 2011, 2012 MediaGoblin contributors.  See AUTHORS.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import argparse
import os

from mediagoblin import mg_globals
from mediagoblin.db.models import MediaEntry
from mediagoblin.gmg_commands import util as commands_util
from mediagoblin.submit.lib import run_process_media
from mediagoblin.tools.translate import lazy_pass_to_ugettext as _
from mediagoblin.tools.pluginapi import hook_handle
from mediagoblin.processing import (
    ProcessorDoesNotExist, ProcessorNotEligible,
    get_entry_and_processing_manager, get_processing_manager_for_type)


def reprocess_parser_setup(subparser):
    subparser.add_argument(
        '--celery',
        action='store_true',
        help="Don't process eagerly, pass off to celery")

    subparsers = subparser.add_subparsers(dest="reprocess_subcommand")

    ###################
    # available command
    ###################
    available_parser = subparsers.add_parser(
        "available",
        help="Find out what actions are available for this media")
    
    available_parser.add_argument(
        "id_or_type",
        help="Media id or media type to check")

    available_parser.add_argument(
        "--action-help",
        action="store_true",
        help="List argument help for each action available")

    
    ############################################
    # run command (TODO: and bulk_run command??)
    ############################################
    
    run_parser = subparsers.add_parser(
        "run",
        help="Run a reprocessing on one or more media")

    run_parser.add_argument(
        '--state', '-s',
        help="Reprocess media entries in this state"
             " such as 'failed' or 'processed'")
    run_parser.add_argument(
        '--type', '-t',
        help="The type of media to be reprocessed such as 'video' or 'image'")
    run_parser.add_argument(
        '--thumbnails',
        action="store_true",
        help="Regenerate thumbnails for all processed media")

    run_parser.add_argument(
        'media_id',
        help="The media_entry id(s) you wish to reprocess.")

    run_parser.add_argument(
        'reprocess_command',
        help="The reprocess command you intend to run")

    run_parser.add_argument(
        'reprocess_args',
        nargs=argparse.REMAINDER,
        help="rest of arguments to the reprocessing tool")


    ###############
    # help command?
    ###############



def _set_media_type(args):
    """
    This will verify that all media id's are of the same media_type. If the
    --type flag is set, it will be replaced by the given media id's type.

    If they are trying to process different media types, an Exception will be
    raised.
    """
    if args[0].media_id:
        if len(args[0].media_id) == 1:
            args[0].type = MediaEntry.query.filter_by(id=args[0].media_id[0])\
                .first().media_type.split('.')[-1]

        elif len(args[0].media_id) > 1:
            media_types = []

            for id in args[0].media_id:
                media_types.append(MediaEntry.query.filter_by(id=id).first()
                                   .media_type.split('.')[-1])
            for type in media_types:
                if media_types[0] != type:
                    raise Exception((u'You cannot reprocess different'
                                     ' media_types at the same time.'))

            args[0].type = media_types[0]


def _reprocess_all(args):
    """
    This handles reprocessing if no media_id's are given.
    """
    if not args[0].type:
        # If no media type is given, we can either regenerate all thumbnails,
        # or try to reprocess all failed media

        if args[0].thumbnails:
            if args[0].available:
                print _('Available options for regenerating all processed'
                        ' media thumbnails: \n'
                        '\t --size: max_width max_height'
                        ' (defaults to config specs)')
            else:
                #TODO regenerate all thumbnails
                pass

        # Reprocess all failed media
        elif args[0].state == 'failed':
            if args[0].available:
                print _('\n Available reprocess actions for all failed'
                        ' media_entries: \n \t --initial_processing')
            else:
                #TODO reprocess all failed entries
                pass

        # If here, they didn't set the --type flag and were trying to do
        # something other the generating thumbnails or initial_processing
        else:
            raise Exception(_('You must set --type when trying to reprocess'
                              ' all media_entries, unless you set --state'
                              ' to "failed".'))

    else:
        _run_reprocessing(args)


def _run_reprocessing(args):
    # Are they just asking for the available reprocessing options for the given
    # media?
    if args[0].available:
        if args[0].state == 'failed':
            print _('\n Available reprocess actions for all failed'
                    ' media_entries: \n \t --initial_processing')
        else:
            result = hook_handle(('reprocess_action', args[0].type), args)
            if not result:
                print _('Sorry there is no available reprocessing for {}'
                        ' entries in the {} state'.format(args[0].type,
                                                          args[0].state))
    else:
        # Run media reprocessing
        return hook_handle(('media_reprocess', args[0].type), args)


def _set_media_state(args):
    """
    This will verify that all media id's are in the same state. If the
    --state flag is set, it will be replaced by the given media id's state.

    If they are trying to process different media states, an Exception will be
    raised.
    """
    if args[0].media_id:
        # Only check if we are given media_ids
        if len(args[0].media_id) == 1:
            args[0].state = MediaEntry.query.filter_by(id=args[0].media_id[0])\
                .first().state

        elif len(args[0].media_id) > 1:
            media_states = []

            for id in args[0].media_id:
                media_states.append(MediaEntry.query.filter_by(id=id).first()
                                    .state)

            # Make sure that all media are in the same state
            for state in media_states:
                if state != media_states[0]:
                    raise Exception(_('You can only reprocess media that is in'
                                      ' the same state.'))

            args[0].state = media_states[0]

    # If no state was set, then we will default to the processed state
    if not args[0].state:
        args[0].state = 'processed'


def available(args):
    # Get the media type, either by looking up media id, or by specific type
    try:
        media_id = int(args.id_or_type)
        media_entry, manager = get_entry_and_processing_manager(media_id)
        media_type = media_entry.media_type
    except ValueError:
        media_type = args.id_or_type
        media_entry = None
        manager = get_processing_manager_for_type(media_type)

    if media_entry is None:
        processors = manager.list_all_processors()
    else:
        processors = manager.list_eligible_processors(media_entry)

    print "Available processors:"
    print "====================="
    print ""

    if args.action_help:
        for processor in processors:
            print processor.name
            print "-" * len(processor.name)

            parser = processor.generate_parser()
            parser.print_help()
            print ""

    else:
        for processor in processors:
            if processor.description:
                print " - %s: %s" % (processor.name, processor.description)
            else:
                print " - %s" % processor.name


def run(args):
    media_entry, manager = get_entry_and_processing_manager(args.media_id)

    # TODO: (maybe?) This could probably be handled entirely by the
    # processor class...
    try:
        processor_class = manager.get_processor(
            args.reprocess_command, media_entry)
    except ProcessorDoesNotExist:
        print 'No such processor "%s" for media with id "%s"' % (
            args.reprocess_command, media_entry.id)
        return
    except ProcessorNotEligible:
        print 'Processor "%s" exists but media "%s" is not eligible' % (
            args.reprocess_command, media_entry.id)
        return

    reprocess_parser = processor_class.generate_parser()
    reprocess_args = reprocess_parser.parse_args(args.reprocess_args)
    reprocess_request = processor_class.args_to_request(reprocess_args)
    run_process_media(
        media_entry,
        reprocess_action=args.reprocess_command,
        reprocess_info=reprocess_request)


def reprocess(args):
    # Run eagerly unless explicetly set not to
    if not args.celery:
        os.environ['CELERY_ALWAYS_EAGER'] = 'true'

    commands_util.setup_app(args)

    if args.reprocess_subcommand == "run":
        run(args)

    elif args.reprocess_subcommand == "available":
        available(args)
